"""Tests for models in the sodarcache app"""

from typing import Union

from django.forms.models import model_to_dict

from test_plus.test import TestCase

# Projectroles dependency
from projectroles.models import Project, SODARUser, SODAR_CONSTANTS
from projectroles.tests.test_models import (
    ProjectMixin,
    RoleMixin,
    RoleAssignmentMixin,
)

from sodarcache.models import JSONCacheItem


# SODAR constants
PROJECT_ROLE_OWNER = SODAR_CONSTANTS['PROJECT_ROLE_OWNER']
PROJECT_ROLE_DELEGATE = SODAR_CONSTANTS['PROJECT_ROLE_DELEGATE']
PROJECT_ROLE_CONTRIBUTOR = SODAR_CONSTANTS['PROJECT_ROLE_CONTRIBUTOR']
PROJECT_ROLE_GUEST = SODAR_CONSTANTS['PROJECT_ROLE_GUEST']
PROJECT_TYPE_CATEGORY = SODAR_CONSTANTS['PROJECT_TYPE_CATEGORY']
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']

# Local constants
APP_NAME = 'sodarcache'


class JSONCacheItemMixin:
    """Helper mixin for JSONCacheItem creation"""

    @classmethod
    def make_item(
        cls,
        project: Project,
        app_name: str,
        name: str,
        user: SODARUser,
        data: Union[dict, list],
    ) -> JSONCacheItem:
        """Create JSONCacheItem object"""
        values = {
            'project': project,
            'app_name': app_name,
            'user': user,
            'name': name,
            'data': data,
        }
        return JSONCacheItem.objects.create(**values)


class JSONCacheItemTestBase(
    ProjectMixin, RoleMixin, RoleAssignmentMixin, TestCase
):
    def setUp(self):
        # Make owner user
        self.user_owner = self.make_user('owner')
        # Init roles
        self.init_roles()
        # Init project and assignment
        self.project = self.make_project(
            'TestProject', PROJECT_TYPE_PROJECT, None
        )
        self.assignment_owner = self.make_assignment(
            self.project, self.user_owner, self.role_owner
        )


class TestJSONCacheItem(JSONCacheItemMixin, JSONCacheItemTestBase):
    def setUp(self):
        super().setUp()
        self.item = self.make_item(
            project=self.project,
            app_name=APP_NAME,
            user=self.user_owner,
            name='test_item',
            data={'test_key': 'test_val'},
        )

    def test_initialization(self):
        expected = {
            'id': self.item.pk,
            'project': self.project.pk,
            'app_name': APP_NAME,
            'name': 'test_item',
            'user': self.user_owner.pk,
            'sodar_uuid': self.item.sodar_uuid,
            'data': {'test_key': 'test_val'},
        }
        self.assertEqual(model_to_dict(self.item), expected)

    def test__str__(self):
        expected = 'TestProject: sodarcache: test_item'
        self.assertEqual(str(self.item), expected)

    def test__repr__(self):
        expected = "JSONCacheItem('TestProject', 'sodarcache', 'test_item')"
        self.assertEqual(repr(self.item), expected)

    def test__repr__no_project(self):
        """Test __repr__() with no project"""
        new_item = self.make_item(
            project=None,
            app_name=APP_NAME,
            user=self.user_owner,
            name='test_item2',
            data={'test_key': 'test_val'},
        )
        expected = "JSONCacheItem('N/A', 'sodarcache', 'test_item2')"
        self.assertEqual(repr(new_item), expected)



"""Tests for UI view permissions in the siteinfo app"""

from django.test import override_settings
from django.urls import reverse

# Projectroles dependency
from projectroles.tests.test_permissions import SiteAppPermissionTestBase


class TestSiteInfoViewPermissions(SiteAppPermissionTestBase):
    """Tests for SiteInfoView permissions"""

    def setUp(self):
        super().setUp()
        self.url = reverse('siteinfo:info')

    def test_site_info(self):
        """Test SiteInfoView GET"""
        good_users = [self.superuser]
        bad_users = [self.anonymous, self.regular_user]
        self.assert_response(self.url, good_users, 200)
        self.assert_response(self.url, bad_users, 302)

    @override_settings(PROJECTROLES_ALLOW_ANONYMOUS=True)
    def test_site_info_anon(self):
        """Test GET with anonymous access"""
        self.assert_response(self.url, self.anonymous, 302)



# Generated by Django 4.2.14 on 2024-07-16 09:26

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    replaces = [
        ('filesfolders', '0001_initial'),
        ('filesfolders', '0002_auto_20180411_1758'),
        ('filesfolders', '0003_rename_uuid'),
        ('filesfolders', '0004_update_uuid'),
        ('filesfolders', '0005_alter_foreignkey_fields'),
    ]

    initial = True

    dependencies = [
        ('projectroles', '0028_populate_finder_role'),
        ('projectroles', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FileData',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                ('bytes', models.TextField()),
                ('file_name', models.CharField(max_length=255)),
                ('content_type', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Folder',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'name',
                    models.CharField(
                        help_text='Name for the object', max_length=255
                    ),
                ),
                (
                    'date_modified',
                    models.DateTimeField(
                        auto_now=True, help_text='DateTime of last modification'
                    ),
                ),
                (
                    'flag',
                    models.CharField(
                        blank=True,
                        choices=[
                            ('FLAG', 'Flagged'),
                            ('FLAG_HEART', 'Flagged (Heart)'),
                            ('IMPORTANT', 'Important'),
                            ('REVOKED', 'Revoked'),
                            ('SUPERSEDED', 'Superseded'),
                        ],
                        help_text='Flag for highlighting the item (optional)',
                        max_length=64,
                        null=True,
                    ),
                ),
                (
                    'description',
                    models.CharField(
                        blank=True,
                        help_text='Description (optional)',
                        max_length=255,
                    ),
                ),
                (
                    'sodar_uuid',
                    models.UUIDField(
                        default=uuid.uuid4,
                        help_text='Filesfolders SODAR UUID',
                        unique=True,
                    ),
                ),
                (
                    'folder',
                    models.ForeignKey(
                        blank=True,
                        help_text='Folder under which object exists (null if '
                        'root folder)',
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='%(app_label)s_%(class)s_children',
                        to='filesfolders.folder',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        help_text='User who owns the object',
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    'project',
                    models.ForeignKey(
                        help_text='Project in which the object belongs',
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='%(app_label)s_%(class)s_objects',
                        to='projectroles.project',
                    ),
                ),
            ],
            options={
                'ordering': ['project', 'name'],
                'unique_together': {('project', 'folder', 'name')},
            },
        ),
        migrations.CreateModel(
            name='File',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'name',
                    models.CharField(
                        help_text='Name for the object', max_length=255
                    ),
                ),
                (
                    'date_modified',
                    models.DateTimeField(
                        auto_now=True, help_text='DateTime of last modification'
                    ),
                ),
                (
                    'flag',
                    models.CharField(
                        blank=True,
                        choices=[
                            ('FLAG', 'Flagged'),
                            ('FLAG_HEART', 'Flagged (Heart)'),
                            ('IMPORTANT', 'Important'),
                            ('REVOKED', 'Revoked'),
                            ('SUPERSEDED', 'Superseded'),
                        ],
                        help_text='Flag for highlighting the item (optional)',
                        max_length=64,
                        null=True,
                    ),
                ),
                (
                    'description',
                    models.CharField(
                        blank=True,
                        help_text='Description (optional)',
                        max_length=255,
                    ),
                ),
                (
                    'sodar_uuid',
                    models.UUIDField(
                        default=uuid.uuid4,
                        help_text='Filesfolders SODAR UUID',
                        unique=True,
                    ),
                ),
                (
                    'file',
                    models.FileField(
                        blank=True,
                        help_text='Uploaded file',
                        null=True,
                        upload_to='filesfolders.FileData/bytes/file_name/'
                        'content_type',
                    ),
                ),
                (
                    'public_url',
                    models.BooleanField(
                        default=False,
                        help_text='Allow providing a public URL for the file',
                    ),
                ),
                (
                    'secret',
                    models.CharField(
                        help_text='Secret string for creating public URL',
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    'folder',
                    models.ForeignKey(
                        blank=True,
                        help_text='Folder under which object exists (null if '
                        'root folder)',
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='%(app_label)s_%(class)s_children',
                        to='filesfolders.folder',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        help_text='User who owns the object',
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    'project',
                    models.ForeignKey(
                        help_text='Project in which the object belongs',
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='%(app_label)s_%(class)s_objects',
                        to='projectroles.project',
                    ),
                ),
            ],
            options={
                'ordering': ['folder', 'name'],
                'unique_together': {('project', 'folder', 'name')},
            },
        ),
        migrations.CreateModel(
            name='HyperLink',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'name',
                    models.CharField(
                        help_text='Name for the object', max_length=255
                    ),
                ),
                (
                    'date_modified',
                    models.DateTimeField(
                        auto_now=True, help_text='DateTime of last modification'
                    ),
                ),
                (
                    'flag',
                    models.CharField(
                        blank=True,
                        choices=[
                            ('FLAG', 'Flagged'),
                            ('FLAG_HEART', 'Flagged (Heart)'),
                            ('IMPORTANT', 'Important'),
                            ('REVOKED', 'Revoked'),
                            ('SUPERSEDED', 'Superseded'),
                        ],
                        help_text='Flag for highlighting the item (optional)',
                        max_length=64,
                        null=True,
                    ),
                ),
                (
                    'description',
                    models.CharField(
                        blank=True,
                        help_text='Description (optional)',
                        max_length=255,
                    ),
                ),
                (
                    'sodar_uuid',
                    models.UUIDField(
                        default=uuid.uuid4,
                        help_text='Filesfolders SODAR UUID',
                        unique=True,
                    ),
                ),
                (
                    'url',
                    models.URLField(
                        help_text='URL for the link', max_length=2000
                    ),
                ),
                (
                    'folder',
                    models.ForeignKey(
                        blank=True,
                        help_text='Folder under which object exists (null if '
                        'root folder)',
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='%(app_label)s_%(class)s_children',
                        to='filesfolders.folder',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        help_text='User who owns the object',
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    'project',
                    models.ForeignKey(
                        help_text='Project in which the object belongs',
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='%(app_label)s_%(class)s_objects',
                        to='projectroles.project',
                    ),
                ),
            ],
            options={
                'ordering': ['folder', 'name'],
                'unique_together': {('project', 'folder', 'name')},
            },
        ),
    ]



"""Django signals for the projectroles app"""

import logging

from django.conf import settings
from django.contrib.auth.signals import (
    user_logged_in,
    user_logged_out,
    user_login_failed,
)

from projectroles.models import AUTH_PROVIDER_OIDC


logger = logging.getLogger(__name__)


# User signals -----------------------------------------------------------------


def handle_ldap_login(sender, user, **kwargs):
    """Signal for LDAP login handling"""
    try:
        if hasattr(user, 'ldap_username'):
            logger.debug('Updating LDAP user..')
            user.update_ldap_username()
            user.update_full_name()
    except Exception as ex:
        logger.error(f'Exception in handle_ldap_login(): {ex}')
        if settings.DEBUG:
            raise ex


def handle_oidc_login(sender, user, **kwargs):
    """Signal for OIDC login handling"""
    social_auth = getattr(user, 'social_auth', None)
    if not social_auth:
        return
    try:
        social_auth = social_auth.first()
        if social_auth and social_auth.provider == AUTH_PROVIDER_OIDC:
            logger.debug('Updating OIDC user..')
            user.update_full_name()
    except Exception as ex:
        logger.error(f'Exception in handle_oidc_login(): {ex}')
        if settings.DEBUG:
            raise ex


def assign_user_group(sender, user, **kwargs):
    """Signal for user group assignment"""
    try:
        user.set_group()
    except Exception as ex:
        logger.error(f'Exception in assign_user_group(): {ex}')
        if settings.DEBUG:
            raise ex


def log_user_login(sender, user, **kwargs):
    """Signal for logging user login"""
    logger.info(f'User logged in: {user.username}')


def log_user_logout(sender, user, **kwargs):
    """Signal for logging user logout"""
    if user:
        logger.info(f'User logged out: {user.username}')


def log_user_login_failure(sender, credentials, **kwargs):
    """Signal for user login failure"""
    logger.info('User login failed: {}'.format(credentials.get('username')))


user_logged_in.connect(handle_ldap_login)
user_logged_in.connect(handle_oidc_login)
user_logged_in.connect(assign_user_group)
user_logged_in.connect(log_user_login)
user_logged_out.connect(log_user_logout)
user_login_failed.connect(log_user_login_failure)



"""Model tests for the appalerts app"""

from typing import Optional

from django.core.exceptions import ValidationError
from django.forms.models import model_to_dict
from django.urls import reverse

from djangoplugins.models import Plugin, PluginPoint
from test_plus.test import TestCase

# Projectroles dependency
from projectroles.models import Project, SODARUser, SODAR_CONSTANTS
from projectroles.tests.test_models import ProjectMixin

from appalerts.models import AppAlert


# SODAR constants
PROJECT_TYPE_PROJECT = SODAR_CONSTANTS['PROJECT_TYPE_PROJECT']

# Local constants
APP_NAME_FF = 'filesfolders'
ALERT_NAME = 'test_alert'
ALERT_MSG = 'Test alert message'
ALERT_LEVEL = 'INFO'


class AppAlertMixin:
    """Mixin for AppAlert test helpers"""

    @classmethod
    def make_app_alert(
        cls,
        app_plugin: Optional[PluginPoint] = None,
        alert_name: str = ALERT_NAME,
        user: Optional[SODARUser] = None,
        message: str = ALERT_MSG,
        level: str = ALERT_LEVEL,
        active: bool = True,
        url: Optional[str] = None,
        project: Optional[Project] = None,
    ) -> AppAlert:
        """Create AppAlert object"""
        return AppAlert.objects.create(
            app_plugin=app_plugin,
            alert_name=alert_name,
            user=user,
            message=message,
            level=level,
            active=active,
            url=url,
            project=project,
        )


class TestAppAlert(AppAlertMixin, ProjectMixin, TestCase):
    """Tests for the AppAlert model"""

    def setUp(self):
        self.project = self.make_project(
            title='TestProject', type=PROJECT_TYPE_PROJECT, parent=None
        )
        self.project_url = reverse(
            'projectroles:detail',
            kwargs={'project': str(self.project.sodar_uuid)},
        )
        self.user = self.make_user()

    def test_initialization(self):
        """Test AppAlert initialization"""
        alert = self.make_app_alert(
            user=self.user,
            url=self.project_url,
            project=self.project,
        )
        expected = {
            'id': alert.pk,
            'app_plugin': None,
            'alert_name': ALERT_NAME,
            'user': self.user.pk,
            'message': ALERT_MSG,
            'level': ALERT_LEVEL,
            'active': True,
            'url': '/project/' + str(self.project.sodar_uuid),
            'project': self.project.pk,
            'sodar_uuid': alert.sodar_uuid,
        }
        model_dict = model_to_dict(alert)
        self.assertEqual(model_dict, expected)

    def test_initialization_plugin(self):
        """Test AppAlert initialization with app plugin"""
        app_plugin = Plugin.objects.get(name=APP_NAME_FF)
        alert = self.make_app_alert(
            app_plugin=app_plugin,
            user=self.user,
            url=self.project_url,
            project=self.project,
        )
        expected = {
            'id': alert.pk,
            'app_plugin': app_plugin.pk,
            'alert_name': ALERT_NAME,
            'user': self.user.pk,
            'message': ALERT_MSG,
            'level': ALERT_LEVEL,
            'active': True,
            'url': '/project/' + str(self.project.sodar_uuid),
            'project': self.project.pk,
            'sodar_uuid': alert.sodar_uuid,
        }
        model_dict = model_to_dict(alert)
        self.assertEqual(model_dict, expected)

    def test_str(self):
        """Test AppAlert __str__()"""
        alert = self.make_app_alert(
            user=self.user,
            url=self.project_url,
            project=self.project,
        )
        expected = f'projectroles / {ALERT_NAME} / {self.user.username}'
        self.assertEqual(str(alert), expected)

    def test_str_plugin(self):
        """Test AppAlert __str__() with app plugin"""
        app_plugin = Plugin.objects.get(name=APP_NAME_FF)
        alert = self.make_app_alert(
            app_plugin=app_plugin,
            user=self.user,
            url=self.project_url,
            project=self.project,
        )
        expected = f'filesfolders / {ALERT_NAME} / {self.user.username}'
        self.assertEqual(str(alert), expected)

    def test_repr(self):
        """Test AppAlert __repr__()"""
        alert = self.make_app_alert(
            user=self.user,
            url=self.project_url,
            project=self.project,
        )
        expected = "AppAlert('projectroles', '{}', '{}', '{}')".format(
            ALERT_NAME, self.user.username, self.project.title
        )
        self.assertEqual(repr(alert), expected)

    def test_repr_plugin(self):
        """Test AppAlert __repr__() with app plugin"""
        app_plugin = Plugin.objects.get(name=APP_NAME_FF)
        alert = self.make_app_alert(
            app_plugin=app_plugin,
            user=self.user,
            url=self.project_url,
            project=self.project,
        )
        expected = "AppAlert('filesfolders', '{}', '{}', '{}')".format(
            ALERT_NAME, self.user.username, self.project.title
        )
        self.assertEqual(repr(alert), expected)

    def test_validate_level(self):
        """Test level validation"""
        with self.assertRaises(ValidationError):
            self.make_app_alert(
                user=self.user,
                level='not a valid level',
                url=self.project_url,
                project=self.project,
            )
